@@ -17,7 +17,8 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * Author: Shaun McCance  <shaunm@gnome.org>
+ * Authors: Shaun McCance  <shaunm@gnome.org>
+ *          Brent Smith  <gnome@nextreality.net>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -29,6 +30,7 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <libgnome/gnome-program.h>
+#include <libgnome/gnome-init.h>
 
 #include "yelp-utils.h"
 
@@ -40,7 +42,7 @@
 #define d(x)
 #endif
 
-GHashTable *doc_info_table;
+static GHashTable *doc_info_table;
 
 typedef struct {
     gchar       *uri;
@@ -69,11 +71,40 @@ struct _YelpDocInfo {
 static YelpDocType  get_doc_type       (gchar   *uri);
 static gchar *      convert_ghelp_uri  (gchar   *uri);
 
-static gchar *      convert_man_uri    (gchar   *uri);
+static gchar *      convert_man_uri    (gchar   *uri, gboolean trust_uri);
 static gchar *      convert_info_uri   (gchar   *uri);
 
+static gchar *dot_dir = NULL;
+
+static gchar **infopath = NULL;
+static gchar *infopath_d[] = {"/usr/info", "/usr/share/info", 
+			      "/usr/local/info", "/usr/local/share/info", 
+			      NULL};
+
+
+const char *
+yelp_dot_dir (void)
+{
+    if (dot_dir == NULL) {
+	dot_dir = g_build_filename (g_get_home_dir(), GNOME_DOT_GNOME,
+	                            "yelp.d", NULL);
+	
+	if (!g_file_test (dot_dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))
+	    mkdir (dot_dir, 0750);		
+    }
+
+    return dot_dir;
+}
+
+/* @uri:       the uri to interpret for the new YelpDocInfo struct 
+ * @trust_uri: if the uri is absolute and is known to exist,
+ *             then this should be set.  Only makes sense for local
+ *             files.  This is here for performance reasons, adding
+ *             40,000 man pages and accessing the disk for each one
+ *             can get pretty expensive.
+ */
 YelpDocInfo *
-yelp_doc_info_new (const gchar *uri)
+yelp_doc_info_new (const gchar *uri, gboolean trust_uri)
 {
     YelpDocInfo *doc;
     gchar       *doc_uri  = NULL;
@@ -87,10 +118,13 @@ yelp_doc_info_new (const gchar *uri)
     d (g_print ("yelp_doc_info_new\n"));
     d (g_print ("  uri      = \"%s\"\n", uri));
 
-    full_uri =
+    if (trust_uri)
+	full_uri = gnome_vfs_make_uri_from_input (uri);
+    else
+	full_uri =
 	gnome_vfs_make_uri_from_input_with_dirs	(uri,
 						 GNOME_VFS_MAKE_URI_DIR_CURRENT);
-
+    
     if (g_str_has_prefix (full_uri, "file:")) {
 	if ((cur = strchr (full_uri, '#')))
 	    doc_uri = g_strndup (full_uri, cur - full_uri);
@@ -107,7 +141,7 @@ yelp_doc_info_new (const gchar *uri)
 	uri_type = YELP_URI_TYPE_GHELP;
     }
     else if (g_str_has_prefix (full_uri, "man:")) {
-	doc_uri  = convert_man_uri (full_uri);
+	doc_uri  = convert_man_uri (full_uri, trust_uri);
 	doc_type = YELP_DOC_TYPE_MAN;
 	uri_type = YELP_URI_TYPE_MAN;
     }
@@ -157,7 +191,7 @@ yelp_doc_info_new (const gchar *uri)
 }
 
 YelpDocInfo *
-yelp_doc_info_get (const gchar *uri)
+yelp_doc_info_get (const gchar *uri, gboolean trust_uri)
 {
     YelpDocInfo *doc;
     gint i;
@@ -187,7 +221,7 @@ yelp_doc_info_get (const gchar *uri)
     doc = (YelpDocInfo *) g_hash_table_lookup (doc_info_table, doc_uri);
 
     if (!doc) {
-	doc = yelp_doc_info_new (doc_uri);
+	doc = yelp_doc_info_new (uri, trust_uri);
 	if (doc && doc->type != YELP_DOC_TYPE_EXTERNAL) {
 	    YelpDocInfo *old_doc = NULL;
 
@@ -477,7 +511,10 @@ yelp_uri_get_fragment (const gchar *uri)
 	if ((cur = strchr (uri, ':')))
 	    if (*(++cur) != '\0')
 		frag_id = g_strdup (cur);
-
+    if (g_str_has_prefix (uri, "info:"))
+	if ((cur = strchr (uri, ')')))
+	    if (*(++cur) != '\0')
+		frag_id = g_strdup (cur);
     if ((cur = strchr (uri, '#')))
 	if (*(++cur) != '\0') {
 	    if (frag_id)
@@ -520,7 +557,7 @@ get_doc_type (gchar *uri)
     if (mime_type == NULL)
 	return YELP_DOC_TYPE_ERROR;
 
-    if (g_str_equal (mime_type, "text/xml"))
+    if (g_str_equal (mime_type, "text/xml") || g_str_equal (mime_type, "application/docbook+xml") || g_str_equal (mime_type, "application/xml"))
 	type = YELP_DOC_TYPE_DOCBOOK_XML;
     else if (g_str_equal (mime_type, "text/sgml"))
 	type = YELP_DOC_TYPE_DOCBOOK_SGML;
@@ -687,7 +724,7 @@ convert_ghelp_uri (gchar *uri)
 }
 
 static gchar *
-convert_man_uri (gchar *uri)
+convert_man_uri (gchar *uri, gboolean trust_uri)
 {
     gchar *path, *cur;
     gchar *doc_uri  = NULL;
@@ -713,7 +750,9 @@ convert_man_uri (gchar *uri)
 
     /* An absolute file path after man: */
     if (path[0] == '/') {
-	if (g_file_test (path, G_FILE_TEST_IS_REGULAR))
+	if (trust_uri)
+	    doc_uri = g_strconcat ("file://", path, NULL);
+	else if (g_file_test (path, G_FILE_TEST_IS_REGULAR))
 	    doc_uri = g_strconcat ("file://", path, NULL);
 	goto done;
     }
@@ -803,36 +842,9 @@ convert_man_uri (gchar *uri)
     return doc_uri;
 }
 
-static gchar *
-convert_info_uri (gchar   *uri)
+gchar **
+yelp_get_info_paths ( )
 {
-    gchar *path, *cur;
-    gchar *doc_uri  = NULL;
-    gchar *info_name = NULL;
-    gchar *info_dot_info = NULL;
-
-    static gchar **infopath = NULL;
-    static gchar *infopath_d[] = {"/usr/info", "/usr/share/info", NULL};
-
-    gint i;
-
-    GDir  *dir;
-    const gchar *filename;
-    gchar *pattern;
-    GPatternSpec *pspec = NULL;
-
-    if ((path = strchr(uri, ':')))
-	path++;
-    else
-	goto done;
-
-    /* An absolute path after info: */
-    if (path[0] == '/') {
-	if (g_file_test (path, G_FILE_TEST_IS_REGULAR))
-	    doc_uri = g_strconcat ("file://", path, NULL);
-	goto done;
-    }
-
     /* Get the infopath, either from the INFOPATH envar,
        or from the default infopath_d.
     */
@@ -849,6 +861,41 @@ convert_info_uri (gchar   *uri)
 	}
     }
 
+
+    return infopath;
+}
+
+
+static gchar *
+convert_info_uri (gchar   *uri)
+{
+    gchar *path, *cur;
+    gchar *doc_uri  = NULL;
+    gchar *info_name = NULL;
+    gchar *info_dot_info = NULL;
+    gchar **infopaths = NULL;
+
+    gint i;
+
+    GDir  *dir;
+    const gchar *filename;
+    gchar *pattern;
+    GPatternSpec *pspec = NULL;
+    GPatternSpec *pspec1 = NULL;
+
+    if ((path = strchr(uri, ':')))
+	path++;
+    else
+	goto done;
+
+    /* An absolute path after info: */
+    if (path[0] == '/') {
+	if (g_file_test (path, G_FILE_TEST_IS_REGULAR))
+	    doc_uri = g_strconcat ("file://", path, NULL);
+	goto done;
+    }
+
+
     /* The URI is one of the following:
        info:info_name
        info:info_name#node
@@ -874,15 +921,22 @@ convert_info_uri (gchar   *uri)
     pattern = g_strdup_printf ("%s.info.*", info_name);
     pspec = g_pattern_spec_new (pattern);
     g_free (pattern);
+    pattern = g_strdup_printf ("%s.?z*", info_name);
+    pspec1 = g_pattern_spec_new (pattern);
+    g_free (pattern);
 
     info_dot_info = g_strconcat (info_name, ".info", NULL);
 
-    for (i = 0; infopath[i]; i++) {
+    infopaths = yelp_get_info_paths ();
+
+    for (i = 0; infopaths[i]; i++) {
 	dir = g_dir_open (infopath[i], 0, NULL);
 	if (dir) {
 	    while ((filename = g_dir_read_name (dir))) {
 		if (g_str_equal (info_dot_info, filename)  ||
-		    g_pattern_match_string (pspec, filename) ) {
+		    g_pattern_match_string (pspec, filename) ||
+		    g_pattern_match_string (pspec1, filename) ||
+		    g_str_equal (info_name, filename)) {
 		    doc_uri = g_strconcat ("file://",
 					   infopath[i], "/",
 					   filename,
@@ -898,6 +952,8 @@ convert_info_uri (gchar   *uri)
  done:
     if (pspec)
 	g_pattern_spec_free (pspec);
+    if (pspec1)
+	g_pattern_spec_free (pspec1);
     g_free (info_dot_info);
     g_free (info_name);
     return doc_uri;
